@@ -38,12 +38,11 @@
 #include <stdlib.h>
 #include <sys/time.h>
 
+#include "dvb-fe-priv.h"
 #include <libdvbv5/dvb-scan.h>
 #include <libdvbv5/dvb-frontend.h>
 #include <libdvbv5/descriptors.h>
-#include "parse_string.h"
 #include <libdvbv5/crc32.h>
-#include <libdvbv5/dvb-fe.h>
 #include <libdvbv5/dvb-file.h>
 #include <libdvbv5/dvb-scan.h>
 #include <libdvbv5/dvb-log.h>
@@ -63,7 +62,7 @@
 #include <libdvbv5/desc_t2_delivery.h>
 #include <libdvbv5/desc_sat.h>
 
-static int poll(struct dvb_v5_fe_parms *parms, int fd, unsigned int seconds)
+static int dvb_poll(struct dvb_v5_fe_parms_priv *parms, int fd, unsigned int seconds)
 {
 	fd_set set;
 	struct timeval timeout;
@@ -79,7 +78,7 @@ static int poll(struct dvb_v5_fe_parms *parms, int fd, unsigned int seconds)
 
 	/* `select' logfuncreturns 0 if timeout, 1 if input available, -1 if error. */
 	do ret = select (FD_SETSIZE, &set, NULL, NULL, &timeout);
-	while (!parms->abort && ret == -1 && errno == EINTR);
+	while (!parms->p.abort && ret == -1 && errno == EINTR);
 
 	return ret;
 }
@@ -134,7 +133,7 @@ struct dvb_table_filter_priv {
 	int done;
 };
 
-static int dvb_parse_section_alloc(struct dvb_v5_fe_parms *parms,
+static int dvb_parse_section_alloc(struct dvb_v5_fe_parms_priv *parms,
 				   struct dvb_table_filter *sect)
 {
 	struct dvb_table_filter_priv *priv;
@@ -166,7 +165,7 @@ void dvb_table_filter_free(struct dvb_table_filter *sect)
 	}
 }
 
-static int dvb_parse_section(struct dvb_v5_fe_parms *parms,
+static int dvb_parse_section(struct dvb_v5_fe_parms_priv *parms,
 			     struct dvb_table_filter *sect,
 			     const uint8_t *buf, ssize_t buf_length)
 {
@@ -177,7 +176,7 @@ static int dvb_parse_section(struct dvb_v5_fe_parms *parms,
 	memcpy(&h, buf, sizeof(struct dvb_table_header));
 	dvb_table_header_init(&h);
 
-	if (parms->verbose)
+	if (parms->p.verbose)
 		dvb_log("%s: received table 0x%02x, TS ID 0x%04x, section %d/%d",
 			__func__, h.table_id, h.id, h.section_id, h.last_section);
 
@@ -221,8 +220,9 @@ static int dvb_parse_section(struct dvb_v5_fe_parms *parms,
 		set_bit(h.section_id, priv->is_read_bits);
 
 	if (dvb_table_initializers[tid])
-		dvb_table_initializers[tid](parms, buf, buf_length - DVB_CRC_SIZE,
-						 sect->table);
+		dvb_table_initializers[tid](&parms->p, buf,
+					    buf_length - DVB_CRC_SIZE,
+					    sect->table);
 	else
 		dvb_logerr("%s: no initializer for table %d",
 			   __func__, tid);
@@ -238,10 +238,11 @@ static int dvb_parse_section(struct dvb_v5_fe_parms *parms,
 	return 1;
 }
 
-int dvb_read_sections(struct dvb_v5_fe_parms *parms, int dmx_fd,
+int dvb_read_sections(struct dvb_v5_fe_parms *__p, int dmx_fd,
 			     struct dvb_table_filter *sect,
 			     unsigned timeout)
 {
+	struct dvb_v5_fe_parms_priv *parms = (void *)__p;
 	int ret;
 	uint8_t *buf = NULL;
 	uint8_t mask = 0xff;
@@ -256,7 +257,7 @@ int dvb_read_sections(struct dvb_v5_fe_parms *parms, int dmx_fd,
 		dvb_dmx_stop(dmx_fd);
 		return -1;
 	}
-	if (parms->verbose)
+	if (parms->p.verbose)
 		dvb_log("%s: waiting for table ID 0x%02x, program ID 0x%02x",
 			__func__, sect->tid, sect->pid);
 
@@ -275,10 +276,10 @@ int dvb_read_sections(struct dvb_v5_fe_parms *parms, int dmx_fd,
 		ssize_t buf_length = 0;
 
 		do {
-			available = poll(parms, dmx_fd, timeout);
+			available = dvb_poll(parms, dmx_fd, timeout);
 		} while (available < 0 && errno == EOVERFLOW);
 
-		if (parms->abort) {
+		if (parms->p.abort) {
 			ret = 0;
 			break;
 		}
@@ -300,7 +301,7 @@ int dvb_read_sections(struct dvb_v5_fe_parms *parms, int dmx_fd,
 			break;
 		}
 
-		crc = crc32(buf, buf_length, 0xFFFFFFFF);
+		crc = dvb_crc32(buf, buf_length, 0xFFFFFFFF);
 		if (crc != 0) {
 			dvb_logerr("%s: crc error", __func__);
 			ret = -3;
@@ -335,8 +336,7 @@ int dvb_read_section_with_id(struct dvb_v5_fe_parms *parms, int dmx_fd,
 	return dvb_read_sections(parms, dmx_fd, &tab, timeout);
 }
 
-struct dvb_v5_descriptors *dvb_scan_alloc_handler_table(uint32_t delivery_system,
-						       int verbose)
+struct dvb_v5_descriptors *dvb_scan_alloc_handler_table(uint32_t delivery_system)
 {
 	struct dvb_v5_descriptors *dvb_scan_handler;
 
@@ -344,7 +344,6 @@ struct dvb_v5_descriptors *dvb_scan_alloc_handler_table(uint32_t delivery_system
 	if (!dvb_scan_handler)
 		return NULL;
 
-	dvb_scan_handler->verbose = verbose;
 	dvb_scan_handler->delivery_system = delivery_system;
 
 	return dvb_scan_handler;
@@ -372,12 +371,13 @@ void dvb_scan_free_handler_table(struct dvb_v5_descriptors *dvb_scan_handler)
 	free(dvb_scan_handler);
 }
 
-struct dvb_v5_descriptors *dvb_get_ts_tables(struct dvb_v5_fe_parms *parms,
+struct dvb_v5_descriptors *dvb_get_ts_tables(struct dvb_v5_fe_parms *__p,
 					     int dmx_fd,
 					     uint32_t delivery_system,
 					     unsigned other_nit,
 					     unsigned timeout_multiply)
 {
+	struct dvb_v5_fe_parms_priv *parms = (void *)__p;
 	int rc;
 	unsigned pat_pmt_time, sdt_time, nit_time, vct_time;
 	int atsc_filter = 0;
@@ -385,7 +385,7 @@ struct dvb_v5_descriptors *dvb_get_ts_tables(struct dvb_v5_fe_parms *parms,
 
 	struct dvb_v5_descriptors *dvb_scan_handler;
 
-	dvb_scan_handler = dvb_scan_alloc_handler_table(delivery_system, parms->verbose);
+	dvb_scan_handler = dvb_scan_alloc_handler_table(delivery_system);
 	if (!dvb_scan_handler)
 		return NULL;
 
@@ -436,32 +436,32 @@ struct dvb_v5_descriptors *dvb_get_ts_tables(struct dvb_v5_fe_parms *parms,
 	};
 
 	/* PAT table */
-	rc = dvb_read_section(parms, dmx_fd,
+	rc = dvb_read_section(&parms->p, dmx_fd,
 			      DVB_TABLE_PAT, DVB_TABLE_PAT_PID,
 			      (void **)&dvb_scan_handler->pat,
 			      pat_pmt_time * timeout_multiply);
-	if (parms->abort)
+	if (parms->p.abort)
 		return dvb_scan_handler;
 	if (rc < 0) {
 		dvb_logerr("error while waiting for PAT table");
 		dvb_scan_free_handler_table(dvb_scan_handler);
 		return NULL;
 	}
-	if (parms->verbose)
-		dvb_table_pat_print(parms, dvb_scan_handler->pat);
+	if (parms->p.verbose)
+		dvb_table_pat_print(&parms->p, dvb_scan_handler->pat);
 
 	/* ATSC-specific VCT table */
 	if (atsc_filter) {
-		rc = dvb_read_section(parms, dmx_fd,
+		rc = dvb_read_section(&parms->p, dmx_fd,
 				      atsc_filter, ATSC_TABLE_VCT_PID,
 				      (void **)&dvb_scan_handler->vct,
 				      vct_time * timeout_multiply);
-		if (parms->abort)
+		if (parms->p.abort)
 			return dvb_scan_handler;
 		if (rc < 0)
 			dvb_logerr("error while waiting for VCT table");
-		else if (parms->verbose)
-			atsc_table_vct_print(parms, dvb_scan_handler->vct);
+		else if (parms->p.verbose)
+			atsc_table_vct_print(&parms->p, dvb_scan_handler->vct);
 	}
 
 	/* PMT tables */
@@ -472,18 +472,20 @@ struct dvb_v5_descriptors *dvb_get_ts_tables(struct dvb_v5_fe_parms *parms,
 		dvb_scan_handler->program[num_pmt].pat_pgm = program;
 
 		if (!program->service_id) {
-			if (parms->verbose)
-				dvb_log("Network PID: 0x%02x", program->pid);
+			if (parms->p.verbose)
+				dvb_log("Program #%d is network PID: 0x%04x",
+					num_pmt, program->pid);
 			num_pmt++;
 			continue;
 		}
-		if (parms->verbose)
-			dvb_log("Program ID %d", program->pid);
-		rc = dvb_read_section(parms, dmx_fd,
+		if (parms->p.verbose)
+			dvb_log("Program #%d ID 0x%04x, service ID 0x%04x",
+				num_pmt, program->pid, program->service_id);
+		rc = dvb_read_section(&parms->p, dmx_fd,
 				      DVB_TABLE_PMT, program->pid,
 				      (void **)&dvb_scan_handler->program[num_pmt].pmt,
 				      pat_pmt_time * timeout_multiply);
-		if (parms->abort) {
+		if (parms->p.abort) {
 			dvb_scan_handler->num_program = num_pmt + 1;
 			return dvb_scan_handler;
 		}
@@ -492,70 +494,71 @@ struct dvb_v5_descriptors *dvb_get_ts_tables(struct dvb_v5_fe_parms *parms,
 				   program->service_id);
 			dvb_scan_handler->program[num_pmt].pmt = NULL;
 		} else {
-			if (parms->verbose)
-				dvb_table_pmt_print(parms, dvb_scan_handler->program[num_pmt].pmt);
+			if (parms->p.verbose)
+				dvb_table_pmt_print(&parms->p,
+						    dvb_scan_handler->program[num_pmt].pmt);
 		}
 		num_pmt++;
 	}
 	dvb_scan_handler->num_program = num_pmt;
 
 	/* NIT table */
-	rc = dvb_read_section(parms, dmx_fd,
+	rc = dvb_read_section(&parms->p, dmx_fd,
 			      DVB_TABLE_NIT, DVB_TABLE_NIT_PID,
 			      (void **)&dvb_scan_handler->nit,
 			      nit_time * timeout_multiply);
-	if (parms->abort)
+	if (parms->p.abort)
 		return dvb_scan_handler;
 	if (rc < 0)
 		dvb_logerr("error while reading the NIT table");
-	else if (parms->verbose)
-		dvb_table_nit_print(parms, dvb_scan_handler->nit);
+	else if (parms->p.verbose)
+		dvb_table_nit_print(&parms->p, dvb_scan_handler->nit);
 
 	/* SDT table */
 	if (!dvb_scan_handler->vct || other_nit) {
-		rc = dvb_read_section(parms, dmx_fd,
+		rc = dvb_read_section(&parms->p, dmx_fd,
 				DVB_TABLE_SDT, DVB_TABLE_SDT_PID,
 				(void **)&dvb_scan_handler->sdt,
 				sdt_time * timeout_multiply);
-		if (parms->abort)
+		if (parms->p.abort)
 			return dvb_scan_handler;
 		if (rc < 0)
 			dvb_logerr("error while reading the SDT table");
-		else if (parms->verbose)
-			dvb_table_sdt_print(parms, dvb_scan_handler->sdt);
+		else if (parms->p.verbose)
+			dvb_table_sdt_print(&parms->p, dvb_scan_handler->sdt);
 	}
 
 	/* NIT/SDT other tables */
 	if (other_nit) {
-		if (parms->verbose)
+		if (parms->p.verbose)
 			dvb_log("Parsing other NIT/SDT");
-		rc = dvb_read_section(parms, dmx_fd,
+		rc = dvb_read_section(&parms->p, dmx_fd,
 				      DVB_TABLE_NIT2, DVB_TABLE_NIT_PID,
 				      (void **)&dvb_scan_handler->nit,
 				      nit_time * timeout_multiply);
-		if (parms->abort)
+		if (parms->p.abort)
 			return dvb_scan_handler;
 		if (rc < 0)
 			dvb_logerr("error while reading the NIT table");
-		else if (parms->verbose)
-			dvb_table_nit_print(parms, dvb_scan_handler->nit);
+		else if (parms->p.verbose)
+			dvb_table_nit_print(&parms->p, dvb_scan_handler->nit);
 
-		rc = dvb_read_section(parms, dmx_fd,
+		rc = dvb_read_section(&parms->p, dmx_fd,
 				DVB_TABLE_SDT2, DVB_TABLE_SDT_PID,
 				(void **)&dvb_scan_handler->sdt,
 				sdt_time * timeout_multiply);
-		if (parms->abort)
+		if (parms->p.abort)
 			return dvb_scan_handler;
 		if (rc < 0)
 			dvb_logerr("error while reading the SDT table");
-		else if (parms->verbose)
-			dvb_table_sdt_print(parms, dvb_scan_handler->sdt);
+		else if (parms->p.verbose)
+			dvb_table_sdt_print(&parms->p, dvb_scan_handler->sdt);
 	}
 
 	return dvb_scan_handler;
 }
 
-struct dvb_v5_descriptors *dvb_scan_transponder(struct dvb_v5_fe_parms *parms,
+struct dvb_v5_descriptors *dvb_scan_transponder(struct dvb_v5_fe_parms *__p,
 					        struct dvb_entry *entry,
 						int dmx_fd,
 					        check_frontend_t *check_frontend,
@@ -563,13 +566,14 @@ struct dvb_v5_descriptors *dvb_scan_transponder(struct dvb_v5_fe_parms *parms,
 						unsigned other_nit,
 						unsigned timeout_multiply)
 {
+	struct dvb_v5_fe_parms_priv *parms = (void *)__p;
 	struct dvb_v5_descriptors *dvb_scan_handler = NULL;
 	uint32_t freq, delsys = SYS_UNDEFINED;
 	int i, rc;
 
 	/* First of all, set the delivery system */
-	retrieve_entry_prop(entry, DTV_DELIVERY_SYSTEM, &delsys);
-	dvb_set_compat_delivery_system(parms, delsys);
+	dvb_retrieve_entry_prop(entry, DTV_DELIVERY_SYSTEM, &delsys);
+	dvb_set_compat_delivery_system(&parms->p, delsys);
 
 	/* Copy data into parms */
 	for (i = 0; i < entry->n_props; i++) {
@@ -579,50 +583,51 @@ struct dvb_v5_descriptors *dvb_scan_transponder(struct dvb_v5_fe_parms *parms,
 		if (entry->props[i].cmd == DTV_DELIVERY_SYSTEM)
 			continue;
 
-		dvb_fe_store_parm(parms, entry->props[i].cmd, data);
+		dvb_fe_store_parm(&parms->p, entry->props[i].cmd, data);
 
-		if (parms->current_sys == SYS_ATSC &&
+		if (parms->p.current_sys == SYS_ATSC &&
 			entry->props[i].cmd == DTV_MODULATION) {
 			if (data != VSB_8 && data != VSB_16)
-				dvb_fe_store_parm(parms,
+				dvb_fe_store_parm(&parms->p,
 						DTV_DELIVERY_SYSTEM,
 						SYS_DVBC_ANNEX_B);
 		}
 	}
 
-	rc = dvb_fe_set_parms(parms);
+	rc = dvb_fe_set_parms(&parms->p);
 	if (rc < 0) {
 		dvb_perror("dvb_fe_set_parms failed");
 		return NULL;
 	}
 
 	/* As the DVB core emulates it, better to always use auto */
-	dvb_fe_store_parm(parms, DTV_INVERSION, INVERSION_AUTO);
+	dvb_fe_store_parm(&parms->p, DTV_INVERSION, INVERSION_AUTO);
 
-	dvb_fe_retrieve_parm(parms, DTV_FREQUENCY, &freq);
-	if (parms->verbose)
-		dvb_fe_prt_parms(parms);
+	dvb_fe_retrieve_parm(&parms->p, DTV_FREQUENCY, &freq);
+	if (parms->p.verbose)
+		dvb_fe_prt_parms(&parms->p);
 
-	rc = check_frontend(args, parms);
+	rc = check_frontend(args, &parms->p);
 	if (rc < 0)
 		return NULL;
 
-	dvb_scan_handler = dvb_get_ts_tables(parms, dmx_fd,
-					parms->current_sys,
+	dvb_scan_handler = dvb_get_ts_tables(&parms->p, dmx_fd,
+					parms->p.current_sys,
 					other_nit,
 					timeout_multiply);
 
 	return dvb_scan_handler;
 }
 
-int estimate_freq_shift(struct dvb_v5_fe_parms *parms)
+int dvb_estimate_freq_shift(struct dvb_v5_fe_parms *__p)
 {
+	struct dvb_v5_fe_parms_priv *parms = (void *)__p;
 	uint32_t shift = 0, bw = 0, symbol_rate, ro;
 	int rolloff = 0;
 	int divisor = 100;
 
 	/* Need to handle only cable/satellite and ATSC standards */
-	switch (parms->current_sys) {
+	switch (parms->p.current_sys) {
 	case SYS_DVBC_ANNEX_A:
 		rolloff = 115;
 		break;
@@ -638,7 +643,7 @@ int estimate_freq_shift(struct dvb_v5_fe_parms *parms)
 	case SYS_DSS:
 	case SYS_TURBO:
 		divisor = 100000;
-		dvb_fe_retrieve_parm(parms, DTV_ROLLOFF, &ro);
+		dvb_fe_retrieve_parm(&parms->p, DTV_ROLLOFF, &ro);
 		switch (ro) {
 		case ROLLOFF_20:
 			rolloff = 120;
@@ -666,11 +671,11 @@ int estimate_freq_shift(struct dvb_v5_fe_parms *parms)
 		 * guard interval there but it should be enough for the
 		 * purposes of estimating a max frequency shift here.
 		 */
-		dvb_fe_retrieve_parm(parms, DTV_SYMBOL_RATE, &symbol_rate);
+		dvb_fe_retrieve_parm(&parms->p, DTV_SYMBOL_RATE, &symbol_rate);
 		bw = (symbol_rate * rolloff) / divisor;
 	}
 	if (!bw)
-		dvb_fe_retrieve_parm(parms, DTV_BANDWIDTH_HZ, &bw);
+		dvb_fe_retrieve_parm(&parms->p, DTV_BANDWIDTH_HZ, &bw);
 
 	/*
 	 * If the max frequency shift between two frequencies is below
@@ -681,8 +686,8 @@ int estimate_freq_shift(struct dvb_v5_fe_parms *parms)
 	return shift;
 }
 
-int new_freq_is_needed(struct dvb_entry *entry, struct dvb_entry *last_entry,
-		       uint32_t freq, enum dvb_sat_polarization pol, int shift)
+int dvb_new_freq_is_needed(struct dvb_entry *entry, struct dvb_entry *last_entry,
+			   uint32_t freq, enum dvb_sat_polarization pol, int shift)
 {
 	int i;
 	uint32_t data;
@@ -704,16 +709,17 @@ int new_freq_is_needed(struct dvb_entry *entry, struct dvb_entry *last_entry,
 	return 1;
 }
 
-struct dvb_entry *dvb_scan_add_entry(struct dvb_v5_fe_parms *parms,
+struct dvb_entry *dvb_scan_add_entry(struct dvb_v5_fe_parms *__p,
 				     struct dvb_entry *first_entry,
 			             struct dvb_entry *entry,
 			             uint32_t freq, uint32_t shift,
 			             enum dvb_sat_polarization pol)
 {
+	struct dvb_v5_fe_parms_priv *parms = (void *)__p;
 	struct dvb_entry *new_entry;
 	int i, n = 2;
 
-	if (!new_freq_is_needed(first_entry, NULL, freq, pol, shift))
+	if (!dvb_new_freq_is_needed(first_entry, NULL, freq, pol, shift))
 		return NULL;
 
 	/* Clone the current entry into a new entry */
@@ -723,7 +729,18 @@ struct dvb_entry *dvb_scan_add_entry(struct dvb_v5_fe_parms *parms,
 		return NULL;
 	}
 
-	memcpy(new_entry, entry, sizeof(*entry));
+	/*
+	 * We can't just copy the entire entry struct, as some strings
+	 * like lnb, channel, vchannel will be freed multiple times.
+	 * So, copy the props and the Satellite parameters only.
+	 */
+	memcpy(new_entry->props, entry->props, sizeof(entry->props));
+	new_entry->n_props = entry->n_props;
+	new_entry->sat_number = entry->sat_number;
+	new_entry->freq_bpf = entry->freq_bpf;
+	new_entry->diseqc_wait = entry->diseqc_wait;
+	if (entry->lnb)
+		new_entry->lnb = strdup(entry->lnb);
 
 	/*
 	 * The frequency should change to the new one. Seek for it and
@@ -786,10 +803,10 @@ static void add_update_nit_dvbc(struct dvb_table_nit *nit,
 	}
 
 	/* Set NIT props for the transponder */
-	store_entry_prop(new, DTV_MODULATION,
-			 dvbc_modulation_table[d->modulation]);
-	store_entry_prop(new, DTV_SYMBOL_RATE, d->symbol_rate);
-	store_entry_prop(new, DTV_INNER_FEC, dvbc_fec_table[d->fec_inner]);
+	dvb_store_entry_prop(new, DTV_MODULATION,
+			     dvbc_modulation_table[d->modulation]);
+	dvb_store_entry_prop(new, DTV_SYMBOL_RATE, d->symbol_rate);
+	dvb_store_entry_prop(new, DTV_INNER_FEC, dvbc_fec_table[d->fec_inner]);
 
 }
 
@@ -807,8 +824,8 @@ static void add_update_nit_isdbt(struct dvb_table_nit *nit,
 		uint32_t mode = isdbt_mode[d->transmission_mode];
 		uint32_t guard = isdbt_interval[d->guard_interval];
 
-		store_entry_prop(tr->entry, DTV_TRANSMISSION_MODE, mode);
-		store_entry_prop(tr->entry, DTV_GUARD_INTERVAL, guard);
+		dvb_store_entry_prop(tr->entry, DTV_TRANSMISSION_MODE, mode);
+		dvb_store_entry_prop(tr->entry, DTV_GUARD_INTERVAL, guard);
 		return;
 	}
 
@@ -837,12 +854,12 @@ static void add_update_nit_1seg(struct dvb_table_nit *nit,
 
 	for (i = 0; i < len; i++) {
 		if (tr->entry->service_id == d->partial_reception[i].service_id) {
-			store_entry_prop(tr->entry,
-					 DTV_ISDBT_PARTIAL_RECEPTION, 1);
+			dvb_store_entry_prop(tr->entry,
+					     DTV_ISDBT_PARTIAL_RECEPTION, 1);
 			return;
 		}
 	}
-	store_entry_prop(tr->entry, DTV_ISDBT_PARTIAL_RECEPTION, 0);
+	dvb_store_entry_prop(tr->entry, DTV_ISDBT_PARTIAL_RECEPTION, 0);
 }
 
 static void add_update_nit_dvbt2(struct dvb_table_nit *nit,
@@ -866,20 +883,20 @@ static void add_update_nit_dvbt2(struct dvb_table_nit *nit,
 		if (tr->entry->service_id != t2->system_id)
 			return;
 
-		store_entry_prop(tr->entry, DTV_DELIVERY_SYSTEM,
-				SYS_DVBT2);
-		store_entry_prop(tr->entry, DTV_STREAM_ID,
-				t2->plp_id);
+		dvb_store_entry_prop(tr->entry, DTV_DELIVERY_SYSTEM,
+				     SYS_DVBT2);
+		dvb_store_entry_prop(tr->entry, DTV_STREAM_ID,
+				     t2->plp_id);
 
 		if (d->length -1 <= 4)
 			return;
 
-		store_entry_prop(tr->entry, DTV_BANDWIDTH_HZ,
-				dvbt2_bw[t2->bandwidth]);
-		store_entry_prop(tr->entry, DTV_GUARD_INTERVAL,
-				dvbt2_interval[t2->guard_interval]);
-		store_entry_prop(tr->entry, DTV_TRANSMISSION_MODE,
-				dvbt2_transmission_mode[t2->transmission_mode]);
+		dvb_store_entry_prop(tr->entry, DTV_BANDWIDTH_HZ,
+				     dvbt2_bw[t2->bandwidth]);
+		dvb_store_entry_prop(tr->entry, DTV_GUARD_INTERVAL,
+				     dvbt2_interval[t2->guard_interval]);
+		dvb_store_entry_prop(tr->entry, DTV_TRANSMISSION_MODE,
+				     dvbt2_transmission_mode[t2->transmission_mode]);
 
 		return;
 	}
@@ -894,16 +911,16 @@ static void add_update_nit_dvbt2(struct dvb_table_nit *nit,
 		if (!new)
 			return;
 
-		store_entry_prop(new, DTV_DELIVERY_SYSTEM,
-				 SYS_DVBT2);
-		store_entry_prop(new, DTV_STREAM_ID,
-				t2->plp_id);
-		store_entry_prop(new, DTV_BANDWIDTH_HZ,
-				dvbt2_bw[t2->bandwidth]);
-		store_entry_prop(new, DTV_GUARD_INTERVAL,
-				dvbt2_interval[t2->guard_interval]);
-		store_entry_prop(new, DTV_TRANSMISSION_MODE,
-				dvbt2_transmission_mode[t2->transmission_mode]);
+		dvb_store_entry_prop(new, DTV_DELIVERY_SYSTEM,
+				     SYS_DVBT2);
+		dvb_store_entry_prop(new, DTV_STREAM_ID,
+				     t2->plp_id);
+		dvb_store_entry_prop(new, DTV_BANDWIDTH_HZ,
+				     dvbt2_bw[t2->bandwidth]);
+		dvb_store_entry_prop(new, DTV_GUARD_INTERVAL,
+				     dvbt2_interval[t2->guard_interval]);
+		dvb_store_entry_prop(new, DTV_TRANSMISSION_MODE,
+				     dvbt2_transmission_mode[t2->transmission_mode]);
 	}
 }
 
@@ -925,19 +942,19 @@ static void add_update_nit_dvbt(struct dvb_table_nit *nit,
 		return;
 
 	/* Set NIT DVB-T props for the transponder */
-	store_entry_prop(new, DTV_MODULATION,
+	dvb_store_entry_prop(new, DTV_MODULATION,
 				dvbt_modulation[d->constellation]);
-	store_entry_prop(new, DTV_BANDWIDTH_HZ,
+	dvb_store_entry_prop(new, DTV_BANDWIDTH_HZ,
 				dvbt_bw[d->bandwidth]);
-	store_entry_prop(new, DTV_CODE_RATE_HP,
+	dvb_store_entry_prop(new, DTV_CODE_RATE_HP,
 				dvbt_code_rate[d->code_rate_hp_stream]);
-	store_entry_prop(new, DTV_CODE_RATE_LP,
+	dvb_store_entry_prop(new, DTV_CODE_RATE_LP,
 				dvbt_code_rate[d->code_rate_lp_stream]);
-	store_entry_prop(new, DTV_GUARD_INTERVAL,
+	dvb_store_entry_prop(new, DTV_GUARD_INTERVAL,
 				dvbt_interval[d->guard_interval]);
-	store_entry_prop(new, DTV_TRANSMISSION_MODE,
+	dvb_store_entry_prop(new, DTV_TRANSMISSION_MODE,
 				dvbt_transmission_mode[d->transmission_mode]);
-	store_entry_prop(new, DTV_HIERARCHY,
+	dvb_store_entry_prop(new, DTV_HIERARCHY,
 				dvbt_hierarchy[d->hierarchy_information]);
 }
 
@@ -966,30 +983,30 @@ static void add_update_nit_dvbs(struct dvb_table_nit *nit,
 
 	/* Set NIT DVB-S props for the transponder */
 
-	store_entry_prop(new, DTV_MODULATION,
-			dvbs_modulation[d->modulation_system]);
-	store_entry_prop(new, DTV_POLARIZATION,
-			dvbs_polarization[d->polarization]);
-	store_entry_prop(new, DTV_SYMBOL_RATE,
-			d->symbol_rate);
-	store_entry_prop(new, DTV_INNER_FEC,
-			dvbs_dvbc_dvbs_freq_inner[d->fec]);
-	store_entry_prop(new, DTV_ROLLOFF,
-				dvbs_rolloff[d->roll_off]);
+	dvb_store_entry_prop(new, DTV_MODULATION,
+			     dvbs_modulation[d->modulation_system]);
+	dvb_store_entry_prop(new, DTV_POLARIZATION,
+			     dvbs_polarization[d->polarization]);
+	dvb_store_entry_prop(new, DTV_SYMBOL_RATE,
+			     d->symbol_rate);
+	dvb_store_entry_prop(new, DTV_INNER_FEC,
+			     dvbs_dvbc_dvbs_freq_inner[d->fec]);
+	dvb_store_entry_prop(new, DTV_ROLLOFF,
+			     dvbs_rolloff[d->roll_off]);
 	if (d->roll_off != 0)
-		store_entry_prop(new, DTV_DELIVERY_SYSTEM,
-					SYS_DVBS2);
+		dvb_store_entry_prop(new, DTV_DELIVERY_SYSTEM,
+				     SYS_DVBS2);
 }
 
 
-void __dvb_add_update_transponders(struct dvb_v5_fe_parms *parms,
-				   struct dvb_v5_descriptors *dvb_scan_handler,
-				   struct dvb_entry *first_entry,
-				   struct dvb_entry *entry,
-				   uint32_t update)
+static void __dvb_add_update_transponders(struct dvb_v5_fe_parms_priv *parms,
+					  struct dvb_v5_descriptors *dvb_scan_handler,
+					  struct dvb_entry *first_entry,
+					  struct dvb_entry *entry,
+					  uint32_t update)
 {
 	struct update_transponders tr = {
-		.parms = parms,
+		.parms = &parms->p,
 		.dvb_scan_handler = dvb_scan_handler,
 		.first_entry = first_entry,
 		.entry = entry,
@@ -1000,38 +1017,44 @@ void __dvb_add_update_transponders(struct dvb_v5_fe_parms *parms,
 	if (!dvb_scan_handler->nit)
 		return;
 
-	tr.shift = estimate_freq_shift(parms);
+	tr.shift = dvb_estimate_freq_shift(&parms->p);
 
-	switch (parms->current_sys) {
+	switch (parms->p.current_sys) {
 	case SYS_DVBC_ANNEX_A:
 	case SYS_DVBC_ANNEX_C:
-		nit_descriptor_handler(parms, dvb_scan_handler->nit,
-				       cable_delivery_system_descriptor,
-				       NULL, add_update_nit_dvbc, &tr);
+		dvb_table_nit_descriptor_handler(
+				&parms->p, dvb_scan_handler->nit,
+				cable_delivery_system_descriptor,
+				NULL, add_update_nit_dvbc, &tr);
 		return;
 	case SYS_ISDBT:
-		nit_descriptor_handler(parms, dvb_scan_handler->nit,
-				       partial_reception_descriptor,
-				       NULL, add_update_nit_1seg, &tr);
-		nit_descriptor_handler(parms, dvb_scan_handler->nit,
-				       ISDBT_delivery_system_descriptor,
-				       NULL, add_update_nit_isdbt, &tr);
+		dvb_table_nit_descriptor_handler(
+				&parms->p, dvb_scan_handler->nit,
+				partial_reception_descriptor,
+				NULL, add_update_nit_1seg, &tr);
+		dvb_table_nit_descriptor_handler(&parms->p,
+				dvb_scan_handler->nit,
+				ISDBT_delivery_system_descriptor,
+				NULL, add_update_nit_isdbt, &tr);
 		return;
 	case SYS_DVBT:
 	case SYS_DVBT2:
-		nit_descriptor_handler(parms, dvb_scan_handler->nit,
-				       extension_descriptor,
-				       NULL, add_update_nit_dvbt2, &tr);
+		dvb_table_nit_descriptor_handler(
+				&parms->p, dvb_scan_handler->nit,
+				extension_descriptor,
+				NULL, add_update_nit_dvbt2, &tr);
 
-		nit_descriptor_handler(parms, dvb_scan_handler->nit,
-				       terrestrial_delivery_system_descriptor,
-				       NULL, add_update_nit_dvbt, &tr);
+		dvb_table_nit_descriptor_handler(
+				&parms->p, dvb_scan_handler->nit,
+				terrestrial_delivery_system_descriptor,
+				NULL, add_update_nit_dvbt, &tr);
 		return;
 	case SYS_DVBS:
 	case SYS_DVBS2:
-		nit_descriptor_handler(parms, dvb_scan_handler->nit,
-				       satellite_delivery_system_descriptor,
-				       NULL, add_update_nit_dvbs, &tr);
+		dvb_table_nit_descriptor_handler(
+				&parms->p, dvb_scan_handler->nit,
+				satellite_delivery_system_descriptor,
+				NULL, add_update_nit_dvbs, &tr);
 		return;
 	default:
 		dvb_log("Transponders detection not implemented for this standard yet.");
@@ -1039,20 +1062,24 @@ void __dvb_add_update_transponders(struct dvb_v5_fe_parms *parms,
 	}
 }
 
-void dvb_add_scaned_transponders(struct dvb_v5_fe_parms *parms,
+void dvb_add_scaned_transponders(struct dvb_v5_fe_parms *__p,
 				 struct dvb_v5_descriptors *dvb_scan_handler,
 				 struct dvb_entry *first_entry,
 				 struct dvb_entry *entry)
 {
+	struct dvb_v5_fe_parms_priv *parms = (void *)__p;
+
 	return __dvb_add_update_transponders(parms, dvb_scan_handler,
 					     first_entry, entry, 0);
 }
 
-void dvb_update_transponders(struct dvb_v5_fe_parms *parms,
+void dvb_update_transponders(struct dvb_v5_fe_parms *__p,
 			     struct dvb_v5_descriptors *dvb_scan_handler,
 			     struct dvb_entry *first_entry,
 			     struct dvb_entry *entry)
 {
+	struct dvb_v5_fe_parms_priv *parms = (void *)__p;
+
 	return __dvb_add_update_transponders(parms, dvb_scan_handler,
 					     first_entry, entry, 1);
 }
